SystemOrganization addCategory: #'iPhone-Events'!

SharedPool subclass: #EventSensorConstants
	instanceVariableNames: ''
	classVariableNames: 'BlueButtonBit CommandKeyBit CtrlKeyBit EventKeyChar EventKeyDown EventKeyUp EventTouchCancelled EventTouchDown EventTouchMoved EventTouchStationary EventTouchUp EventTypeAccelerationData EventTypeComplex EventTypeDragDropFiles EventTypeKeyboard EventTypeLocationData EventTypeMenu EventTypeMouse EventTypeNone EventTypeWindow OptionKeyBit RedButtonBit ShiftKeyBit TouchPhaseBegan TouchPhaseCancelled TouchPhaseEnded TouchPhaseMoved TouchPhaseStationary WindowEventActivated WindowEventClose WindowEventIconise WindowEventMetricChange WindowEventPaint YellowButtonBit'
	poolDictionaries: ''
	category: 'iPhone-Events'!

!EventSensorConstants commentStamp: 'LaurentLaffont 3/15/2011 20:48' prior: 0!
This is a constants, used by EventSensor to identify various event types and theirs contents.

NOTE: A constant values should be kept in sync with VM-side, which are defined in sq.h header file.!

!EventSensorConstants class methodsFor: '*iPhone-Events-override' stamp: 'John M McIntosh 9/11/2010 13:54'!
initialize
	"EventSensorConstants initialize"
	RedButtonBit := 4.
	BlueButtonBit := 2.
	YellowButtonBit := 1.

	ShiftKeyBit := 1.
	CtrlKeyBit := 2.
	OptionKeyBit := 4.
	CommandKeyBit := 8.

	"Types of events"
	EventTypeNone := 0.
	EventTypeMouse := 1.
	EventTypeKeyboard := 2.
	EventTypeDragDropFiles := 3.
	EventTypeMenu := 4.
	EventTypeWindow := 5.
	EventTypeComplex := 6.

	"Press codes for keyboard events"
	EventKeyChar := 0.
	EventKeyDown := 1.
	EventKeyUp := 2.
	
	"types for touch events"
	
	EventTouchDown := 1.
	EventTouchUp := 2.
	EventTouchMoved := 3.
	EventTouchStationary := 4.
	EventTouchCancelled := 5.
	EventTypeAccelerationData := 6.
	EventTypeLocationData := 7.
	
	"Window event action codes"
	WindowEventMetricChange := 1. " size or position of window changed - value1-4 are left/top/right/bottom values "
	WindowEventClose := 2. " window close icon pressed "
	WindowEventIconise := 3. " window iconised  or hidden etc "
	WindowEventActivated :=4. " window made active - some platforms only - do not rely upon this "
	WindowEventPaint := 5. " window area (in value1-4) needs updating. Some platforms do not need to send this, do not rely on it in image "	

	"Apple touch phase constants"

	TouchPhaseBegan := 0.
	TouchPhaseMoved := 1.
	TouchPhaseStationary := 2.
	TouchPhaseEnded := 3.
	TouchPhaseCancelled := 4.! !

!HandMorph methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 14:17'!
generateComplexEvent: firstEvtBuf 
	"Process the complex event in evtBuf, and all other queued complex events"

	| evtBuf |
	evtBuf := firstEvtBuf.
	[evtBuf notNil]
		whileTrue: [
			(evtBuf at: 3) <= EventTouchCancelled
				ifTrue: [self generateTouchEvent: evtBuf].
			evtBuf := Sensor eventQueue nextOrNilSuchThat: [:each |
				each first = EventTypeComplex]].
	^nil! !

!HandMorph methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 14:17'!
generateTouchEvent: evtBuf 
	"Process the touch events in evtBuf. On a touch down event, create a new hand. Otherwise, queue the event for the hand tracking that finger."

	| type touches id touchHand |
	type := evtBuf at: 3.
	touches := evtBuf at: 4.

	touches do: [:touchEvt |
		id := touchEvt at: 11 ifAbsent: [nil].
		id notNil ifTrue: [
			touchHand := nil.
			type = EventTouchDown ifTrue: [
				touchHand := TouchHandMorph new.
				owner addHand: touchHand. 
				touchHand queueEvent: touchEvt.
				touchHand processEvents.	"sets id"
			] ifFalse: [
				touchHand := owner hands detect: [:each |
					(each isKindOf: TouchHandMorph) and: [each id = id]]
						ifNone: [nil].
				touchHand ifNotNil: [
					touchHand queueEvent: touchEvt
				].
			].
		].
	].

	"if all fingers lifted, or touch otherwise cancelled, remove all touch hands"
	({EventTouchUp. EventTouchCancelled} includes: type) ifTrue: [
		owner hands do: [:each |
			(each isKindOf: TouchHandMorph)
				ifTrue: [owner removeHand: each]]].

	^nil
! !

!HandMorph methodsFor: '*iPhone-Events' stamp: 'BertFreudenberg 12/26/2010 13:26'!
isTouch
	^Smalltalk os platformName = 'iPhone'
! !

!HandMorph methodsFor: '*iPhone-Events-override' stamp: 'BertFreudenberg 6/30/2011 14:38'!
mouseTrailFrom: currentBuf 
	"Current event, a mouse event buffer, is about to be processed.  If there are other similar mouse events queued up, then drop them from the queue, and report the positions inbetween."

	| trail similarEvent |
	trail := (Array new: 1) writeStream.
	trail nextPut: currentBuf third @ currentBuf fourth.
	[(similarEvent := Sensor nextEventOrNilSuchThat: [:each |
		each first = currentBuf first	"same event type"
			and: [each fifth = currentBuf fifth "same buttons"
				and: [each sixth = currentBuf sixth "same modifiers"]]
	]) notNil] whileTrue: 
		[trail nextPut: similarEvent third @ similarEvent fourth].
	^trail contents! !

!HandMorph methodsFor: '*iPhone-Events' stamp: 'BertFreudenberg 12/25/2010 15:33'!
processEvents
	"Process user input events from the local input devices."

	| evt evtBuf type hadAny |
	ActiveEvent ifNotNil: 
			["Meaning that we were invoked from within an event response.
		Make sure z-order is up to date"

			self mouseOverHandler processMouseOver: lastMouseEvent].
	hadAny := false.
	[(evtBuf := Sensor nextEvent) isNil] whileFalse: 
			[evt := nil.	"for unknown event types"
			type := evtBuf first.
			type = EventTypeMouse ifTrue: [evt := self generateMouseEvent: evtBuf].
			type = EventTypeKeyboard 
				ifTrue: [evt := self generateKeyboardEvent: evtBuf].
			type = EventTypeDragDropFiles 
				ifTrue: [evt := self generateDropFilesEvent: evtBuf].
			type = EventTypeWindow
				ifTrue:[evt := self generateWindowEvent: evtBuf].				
			type = EventTypeComplex
				ifTrue: [evt := self generateComplexEvent: evtBuf].
			"All other events are ignored"
			(type ~= EventTypeDragDropFiles and: [evt isNil]) ifTrue: [^self].
			evt isNil 
				ifFalse: 
					["Finally, handle it"

					self handleEvent: evt.
					hadAny := true.

					"For better user feedback, return immediately after a mouse event has been processed."
					(evt isMouse and: [evt isMouseWheel not]) ifTrue: [^self]]].
	"note: if we come here we didn't have any mouse events"
	mouseClickState notNil 
		ifTrue: 
			["No mouse events during this cycle. Make sure click states time out accordingly"

			mouseClickState handleEvent: lastMouseEvent asMouseMove from: self].
	hadAny 
		ifFalse: 
			["No pending events. Make sure z-order is up to date"

			self mouseOverHandler processMouseOver: lastMouseEvent]! !

HandMorph subclass: #TouchHandMorph
	instanceVariableNames: 'id queue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iPhone-Events'!

!TouchHandMorph commentStamp: '<historical>' prior: 0!
I'm a HandMorph tracking multi-touch events.

On every touch-down event (one finger down) an instance of me gets added to the world. It tracks that finger's movement until the touch-up event (finger lifted from surface).!

!TouchHandMorph methodsFor: 'accessing' stamp: 'bf 5/19/2010 16:23'!
id
	^id! !

!TouchHandMorph methodsFor: 'accessing' stamp: 'bf 6/7/2010 17:24'!
isTouch
	^true! !

!TouchHandMorph methodsFor: 'event handling' stamp: 'bf 6/6/2010 19:53'!
mouseTrailFrom: mouseEventBuf
	"aggregate all queued touch move events into a single mouse move's trail"
	| trail pos touch prevPos |
	pos := mouseEventBuf third @ mouseEventBuf fourth.
	trail := WriteStream on: (Array new: 1).
	trail nextPut: pos.
	[queue isEmpty] whileFalse: 
		[
			touch := queue first.
			(touch at: 3) < 3		"phase, still touching"
				ifFalse: [^trail contents].
			prevPos := pos.
			pos := (touch at: 7) asInteger @ (touch at: 8) asInteger.
			pos = prevPos
				ifFalse: [trail nextPut: pos].
			queue removeFirst.
		].
	^trail contents! !

!TouchHandMorph methodsFor: 'event handling' stamp: 'bf 6/13/2010 17:59'!
processEvents
	"Process all events queued for this finger."
	queue ifNil: [^self].

	[queue isEmpty] whileFalse: [
		| touch phase pos |
		touch := queue removeFirst.
		phase := touch at: 3.

		id ifNil: [
			"new touch - but ignore if already ended"
			phase < TouchPhaseEnded ifFalse: [^owner removeHand: self].
			id := touch at: 11.
		].

		phase < TouchPhaseEnded ifTrue: [
			pos := (touch at: 7) asInteger @ (touch at: 8) asInteger.
		] ifFalse: [
			pos := (lastEventBuffer at: 3)@(lastEventBuffer at: 4).
		].

		(self generateMouseEvent: {
			EventTypeMouse. 
			touch at: 1. 		"timeStamp"
			pos x.
			pos y. 
			phase < TouchPhaseEnded ifTrue: [MouseEvent redButton] ifFalse: [0].
			0.
			0.
			1 "assume mainWindow"})
				ifNotNilDo: [:evt | self handleEvent: evt].

		phase < TouchPhaseEnded ifFalse: [^owner removeHand: self].
	].



"			interpreterProxy->storePointerofObjectwithValue(0, storageArea, squeakMSTime);
			interpreterProxy->storePointerofObjectwithValue(1, storageArea, timeStamp);
			interpreterProxy->storePointerofObjectwithValue(2, storageArea, phase);
			interpreterProxy->storePointerofObjectwithValue(3, storageArea, tapCount);
			interpreterProxy->storePointerofObjectwithValue(4, storageArea, window);
			interpreterProxy->storePointerofObjectwithValue(5, storageArea, view);
			interpreterProxy->storePointerofObjectwithValue(6, storageArea, locationInViewX);
			interpreterProxy->storePointerofObjectwithValue(7, storageArea, locationInViewY);
			interpreterProxy->storePointerofObjectwithValue(8, storageArea, previousLocationInViewX);
			interpreterProxy->storePointerofObjectwithValue(9, storageArea, previousLocationInViewY); 
			interpreterProxy->storePointerofObjectwithValue(10, storageArea, touchId); 
			
typedef enum {
    UITouchPhaseBegan, = 0         // whenever a finger touches the surface.
    UITouchPhaseMoved,  1          // whenever a finger moves on the surface.
    UITouchPhaseStationary,  2      // whenever a finger is touching the surface but hasn't moved since the previous event.
    UITouchPhaseEnded,     3        // whenever a finger leaves the surface.
    UITouchPhaseCancelled,  4       // whenever a touch doesn't end but we need to stop tracking (e.g. putting device to face)
}
"
! !

!TouchHandMorph methodsFor: 'event handling' stamp: 'bf 5/19/2010 16:32'!
queueEvent: touchEvt
	queue ifNil: [queue := OrderedCollection new].
	queue add: touchEvt! !

!InputEventSensor class methodsFor: '*iPhone-Events' stamp: 'EstebanLorenzano 2/1/2012 19:07'!
initialize

	self instVarIndexFor: 'touchCmdPressed'
		ifAbsent: [self addInstVarNamed: 'touchCmdPressed' ].
	self instVarIndexFor: 'touchContacts'
		ifAbsent: [self addInstVarNamed: 'touchContacts' ].
	(self allClassVarNames includes:  'TouchCmdRect')
		ifFalse: [self addClassVarNamed: 'TouchCmdRect']! !

!InputEventSensor class methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 13:49'!
touchCmdRect
	^TouchCmdRect! !

!InputEventSensor class methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 13:50'!
touchCmdRect: aRectangleOrNil
	"Force cmd bit on keyboard and mouse events while touching inside this rectangle"
	TouchCmdRect := aRectangleOrNil! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'BertFreudenberg 6/29/2011 00:13'!
nextEventOrNilSuchThat: aBlock
	^eventQueue nextOrNilSuchThat: aBlock! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'BertFreudenberg 12/26/2010 15:41'!
processAccelerationData: evt
	| objectiveCAddress obj |
	
	objectiveCAddress := evt at: 4.
	obj := ObjectiveCObject newWithExternalAddress: objectiveCAddress.
	Transcript show: (obj x);space;show:(obj y);space;show:(obj z);cr.
	obj release.
	^nil! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'BertFreudenberg 12/26/2010 15:40'!
processComplexEventType: evt
	(evt at: 3) <= EventTouchCancelled 
		ifTrue: [^self processTouchEventType: evt].
	(evt at: 3) = EventTypeAccelerationData
		ifTrue: [^self processAccelerationData: evt].
	^nil! !

!InputEventSensor methodsFor: '*iPhone-Events-override' stamp: 'John M McIntosh 9/4/2009 18:13'!
processEvent: evt 
	"Process a single event. This method is run at high priority."
	| type |
		
	type := evt at: 1.

	"Treat menu events first"
	type = EventTypeMenu
		ifTrue: [
			self processMenuEvent: evt.
			^nil].

	"Tackle mouse events first"
	type = EventTypeMouse
		ifTrue: [
			"Transmogrify the button state according to the platform's button map definition"
			evt at: 5 put: (ButtonDecodeTable at: (evt at: 5) + 1).
			"Map the mouse buttons depending on modifiers"
			evt at: 5 put: (self mapButtons: (evt at: 5) modifiers: (evt at: 6)).

			"Update state for polling calls"
			mousePosition := (evt at: 3) @ (evt at: 4).
			modifiers := evt at: 6.
			mouseButtons := evt at: 5.

			^evt].
	
	
	"Finally keyboard"
	type = EventTypeKeyboard
		ifTrue: [
			"Sswap ctrl/alt keys if neeeded"
			KeyDecodeTable
				at: {evt at: 3. evt at: 5}
				ifPresent: [:a | 
					evt
						at: 3 put: a first;
						at: 5 put: a second]. 

			"Update state for polling calls"
			modifiers := evt at: 5. 
			^evt].
		
	type = EventTypeComplex 
		ifTrue: [^self processComplexEventType: evt].
			
	"Handle all events other than Keyborad or Mouse."
	^evt.
	! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'EstebanLorenzano 2/8/2012 14:37'!
processTouchEventType: evt
	"Create regular mouse events for one finger. While another finger is held down inside TouchCmdRect, produce blue button events, otherwise red button.
	If more than these one or two fingers touches, queue the unprocessed touch events for consumption by Morphic."

	| unprocessed fingers |
	
	self touchContacts.				"force creation"

	unprocessed := #().
	fingers := (evt at: 4).

	fingers do: [:finger |
		| id phase pos buttons |
		id := finger at: 11.
		phase := finger at: 3.
		pos := (finger at: 7) asInteger @ (finger at: 8) asInteger.

		(evt at: 3) caseOf: {

			"--------------------------------------------------------------------------------------"

			[EventTouchDown]								"New finger down (but ignore if phase is not down)"
				-> [
					phase < TouchPhaseEnded ifTrue: [
						| newContact |
						newContact := Dictionary with: #pos -> pos with: #buttons->0.
						(touchCmdPressed isNil 
							and: [ 
								(TouchCmdRect notNil and: [TouchCmdRect containsPoint: pos]) 
									or: [ fingers size = 2 ] ])
							ifTrue: [touchCmdPressed := id]
							ifFalse: [
								"only process one other finger"
								touchContacts size = (touchCmdPressed ifNil: [0] ifNotNil: [1])
									ifTrue: [
									buttons := touchCmdPressed ifNil: [4 "left"] ifNotNil: [1 "right"].
									newContact at: #buttons put: buttons.
									self handleEvent: {EventTypeMouse. evt at: 2. pos x. pos y. buttons. 0. 0. evt at: 8}]
								ifFalse: [unprocessed := unprocessed copyWith: finger]].
						touchContacts at: id put: newContact].
				].

			"--------------------------------------------------------------------------------------"

			[EventTouchUp]									"Some finger done (might have been lifted before w/ phase=ended)"
				-> [
					 touchContacts at: id ifPresent: [:contact |
						touchCmdPressed = id 
							ifTrue: [ touchCmdPressed := nil ] 
							ifFalse: [
								(contact at: #buttons) > 0 ifTrue: [
									pos := contact at: #pos.
									self handleEvent: {
										EventTypeMouse. 
										evt at: 2. 
										pos x. 
										pos y. 
										0. 
										0.
										0. 
										evt at: 8}.
								] ifFalse: [
									unprocessed := unprocessed copyWith: finger.
								].
							].
						touchContacts removeKey: id].
					].

			"--------------------------------------------------------------------------------------"

			[EventTouchMoved]
				-> [
					touchContacts at: id ifPresent: [:contact |
						phase < TouchPhaseEnded ifTrue: [
							"finger still down"
							buttons := contact at: #buttons.
							buttons > 0 ifTrue: [
								self handleEvent: {EventTypeMouse. evt at: 2. pos x. pos y. buttons. 0. 0. evt at: 8}
							] ifFalse: [
								unprocessed := unprocessed copyWith: finger.
							].
							contact at: #pos put: pos.
						] ifFalse: [
							"finger lifted, simulate mouse-up"
							touchCmdPressed = id ifTrue: [
									touchCmdPressed := nil.
								] ifFalse: [
									(contact at: #buttons) > 0 ifTrue: [
										pos := contact at: #pos.
										self handleEvent: {EventTypeMouse. evt at: 2. pos x. pos y. 0. 0. 0. evt at: 8}.
									] ifFalse: [
										unprocessed := unprocessed copyWith: finger.
									].
								].
							touchContacts removeKey: id]].
					].

			"--------------------------------------------------------------------------------------"

			[EventTouchCancelled]						"Simulate lifting all fingers"
				-> [
					touchContacts keys do: [:each |
						touchCmdPressed = each ifTrue: [
							touchCmdPressed := nil.
						] ifFalse: [
							| eachContact |
							eachContact := touchContacts at: each.
							(eachContact at: #buttons) > 0 ifTrue: [
								pos := eachContact at: #pos.
								self handleEvent: {EventTypeMouse. evt at: 2. pos x. pos y. 0. 0. 0. evt at: 8}.
							] ifFalse: [
								unprocessed := unprocessed copyWith: finger.
							].
						].
						touchContacts removeKey: each].
					].

			"--------------------------------------------------------------------------------------"
		}.
	].

	unprocessed = #() ifFalse: [
		evt at: 4 put: unprocessed.
		self handleEvent: evt].
	^nil
"
typedef struct sqComplexEvent
	{
		int type;			/* type of event;  EventTypeComplex */
		unsigned int timeStamp;	/* time stamp */
		/* the interpretation of the following fields depend on the type  of the event */
		int action;		        /* one of ComplexEventXXX (see below) */
		usqInt objectPointer;	/* used to point to object */
		int unused1;			/*  */
		int unused2;			/*  */
		int unused3;			/*  */
		int windowIndex;	/* host window structure */
	} sqComplexEvent;
"

"			interpreterProxy->storePointerofObjectwithValue(0, storageArea, squeakMSTime);
			interpreterProxy->storePointerofObjectwithValue(1, storageArea, timeStamp);
			interpreterProxy->storePointerofObjectwithValue(2, storageArea, phase);
			interpreterProxy->storePointerofObjectwithValue(3, storageArea, tapCount);
			interpreterProxy->storePointerofObjectwithValue(4, storageArea, window);
			interpreterProxy->storePointerofObjectwithValue(5, storageArea, view);
			interpreterProxy->storePointerofObjectwithValue(6, storageArea, locationInViewX);
			interpreterProxy->storePointerofObjectwithValue(7, storageArea, locationInViewY);
			interpreterProxy->storePointerofObjectwithValue(8, storageArea, previousLocationInViewX);
			interpreterProxy->storePointerofObjectwithValue(9, storageArea, previousLocationInViewY); 
			interpreterProxy->storePointerofObjectwithValue(10, storageArea, touchId); 
			
typedef enum {
    UITouchPhaseBegan, = 0         // whenever a finger touches the surface.
    UITouchPhaseMoved,  1          // whenever a finger moves on the surface.
    UITouchPhaseStationary,  2      // whenever a finger is touching the surface but hasn't moved since the previous event.
    UITouchPhaseEnded,     3        // whenever a finger leaves the surface.
    UITouchPhaseCancelled,  4       // whenever a touch doesn't end but we need to stop tracking (e.g. putting device to face)
}
	EventTouchDown := 1.
	EventTouchUp := 2.
	EventTouchMoved := 3.
	EventTouchStationary := 4.
	EventTouchCancelled := 5.

"
! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 13:54'!
touchCmdPressed
	^touchCmdPressed notNil! !

!InputEventSensor methodsFor: '*iPhone-Events' stamp: 'John M McIntosh 9/11/2010 13:54'!
touchContacts
	^touchContacts ifNil: [touchContacts := Dictionary new].
! !

!FileDirectory class methodsFor: '*iPhone-Events-override' stamp: 'EstebanLorenzano 1/24/2012 15:52'!
iPhoneCurrentDirectory
 	| dfm documentsPath | 
	dfm := (ObjectiveCBridge classObjectForName: #NSFileManager)
		 defaultManager.
	^dfm currentDirectoryPath asString! !

!FileDirectory class methodsFor: '*iPhone-Events-override' stamp: 'EstebanLorenzano 1/10/2012 20:35'!
startUp
	"Establish the platform-specific FileDirectory subclass. Do any platform-specific startup."
	self setDefaultDirectoryClass.
	Smalltalk os platformName = 'iPhone'
		ifTrue: [self setDefaultDirectory: self iPhoneCurrentDirectory ]
		ifFalse: [self setDefaultDirectory: (self dirPathFor: Smalltalk imageName)].
	Smalltalk at: #SecurityManager ifPresent: [:securityManager |
		securityManager default startInUntrustedDirectory ifTrue: [
			self setDefaultDirectory: securityManager default untrustedUserDirectory.
			DefaultDirectory assureExistence]].
	Smalltalk openSourceFiles
! !

!TextMorph methodsFor: '*iphone-events-overrides' stamp: 'AlainPlantec 10/25/2010 17:11'!
keyboardFocusChange: aBoolean 	
	| w ptm |	
	paragraph isNil ifFalse:[paragraph focused: aBoolean].	
	aBoolean 
		ifTrue:["A hand is wanting to send us characters..."
			self hasFocus ifFalse: [self editor	"Forces install"].
			Editor blinkingCursor ifTrue: [ self startBlinking ]] 
		ifFalse:["A hand has clicked elsewhere..."		
			(w := self world) 
				ifNotNil:[w handsDo: [:h | h keyboardFocus == self ifTrue: [^self]].
					"Release control unless some hand is still holding on"			
					self releaseEditor].	
			self stopBlinking].
	self focusChanged! !

!TextMorph methodsFor: '*iphone-events-overrides' stamp: 'EstebanLorenzano 1/24/2012 14:55'!
wantsKeyboardFocus
	| result |
	
	result := super wantsKeyboardFocus.
	result ifTrue: [ IPhonePlatform showKeyboard: true ].
	
	^result! !

Morph subclass: #TouchCmdMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iPhone-Events'!

!TouchCmdMorph commentStamp: '<historical>' prior: 0!
I designate an area that can be used as "Cmd Key" on a multi-touch screen. My bounds are set as EventSensor's touchCmdRect.!

!TouchCmdMorph methodsFor: 'initialize/release' stamp: 'bf 5/14/2010 14:45'!
initialize
	super initialize.
	self color: Color gray.
	self borderColor: Color blue.
	self borderWidth: 4.
! !

!TouchCmdMorph methodsFor: 'initialize/release' stamp: 'bf 5/14/2010 11:28'!
intoWorld: aWorld
	super intoWorld: aWorld.
	aWorld
		when: #aboutToLeaveWorld
		send: #outOfWorld:
		to: self
		with: aWorld.
! !

!TouchCmdMorph methodsFor: 'initialize/release' stamp: 'John M McIntosh 9/11/2010 14:44'!
outOfWorld: aWorld
	super outOfWorld: aWorld.
	InputEventSensor touchCmdRect: nil.
! !

!TouchCmdMorph methodsFor: 'stepping' stamp: 'John M McIntosh 9/11/2010 14:44'!
step
	self color: (Sensor touchCmdPressed
		ifTrue: [Color blue]
		ifFalse: [Color gray]).
	InputEventSensor touchCmdRect: self bounds.

! !

!TouchCmdMorph methodsFor: 'stepping' stamp: 'bf 5/14/2010 10:43'!
stepTime
	^300 "milliseconds"! !

!TouchCmdMorph methodsFor: 'testing' stamp: 'bf 5/14/2010 10:57'!
wantsToBeTopmost
	^true! !

!UIManager methodsFor: '*iphone-events-overrides' stamp: 'EstebanLorenzano 2/22/2012 13:12'!
boot: bootingFromDisk during: aBlock

	(bootingFromDisk and: [ OSPlatform isIPhone not]) ifTrue: [ 
		" we should replace currently active UI manager with non-interactive one during 
		processing startup list "
		Default := StartupUIManager replacing: self.
	].

	aBlock ensure: [ Default := self ].

	self onSnapshot: bootingFromDisk! !

OSPlatform subclass: #IPhonePlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iPhone-Events'!

!IPhonePlatform class methodsFor: 'private' stamp: 'BertFreudenberg 6/28/2011 15:30'!
isActivePlatform
	^ (Smalltalk at: #OSPlatform ifAbsent: [SmalltalkImage current])
		platformName = 'iPhone'! !

!IPhonePlatform class methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 1/24/2012 14:44'!
showKeyboard: aBoolean
	<primitive: 'primitiveShowKeyboard' module: 'TouchPlugin'>
	! !

!IPhonePlatform methodsFor: 'accessing' stamp: 'John M McIntosh 9/4/2009 16:36'!
clipboardClass
	^MacOSClipboard! !

!IPhonePlatform methodsFor: 'accessing' stamp: 'John M McIntosh 9/4/2009 16:36'!
platformFamily
	^#IPhone! !

!OSPlatform class methodsFor: '*iphone-events-extensions' stamp: 'EstebanLorenzano 2/17/2012 11:00'!
isIPhone
	^self platformName = 'iPhone'! !

!SmalltalkImage methodsFor: '*iPhone-Events-overrides' stamp: 'EstebanLorenzano 2/17/2012 11:01'!
logDuring: aMonadicBlock
	" for safe use, if stream is a file, it needs to be closed after use " 
	
	OSPlatform isIPhone
		ifTrue: [ self logToConsole: aMonadicBlock ]
		ifFalse: [ self logToFile: aMonadicBlock ].! !

!SmalltalkImage methodsFor: '*iPhone-Events-overrides' stamp: 'EstebanLorenzano 2/17/2012 11:01'!
logToConsole: aMonadicBlock
	"Write a log to console"
	ObjectiveCBridge callNSLogWithSmalltalkString: '>>BEGIN SMALLTALK LOG<<'.
	(String streamContents: [ :stream | aMonadicBlock value: stream ]) 
		linesDo: [ :line | ObjectiveCBridge callNSLogWithSmalltalkString: line ].
	ObjectiveCBridge callNSLogWithSmalltalkString: '>>END SMALLTALK LOG<<'.
! !

!SmalltalkImage methodsFor: '*iPhone-Events-overrides' stamp: 'EstebanLorenzano 2/17/2012 11:02'!
logToFile: aMonadicBlock
	" for safe use, if stream is a file, it needs to be closed after use " 
	| logStream |	

	[
		logStream := self openLog.
		aMonadicBlock value: logStream.
	] ensure: [ self closeLog: logStream ].

! !

FSPlatformResolver subclass: #FSIOSResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iPhone-Events'!

!FSIOSResolver class methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 1/24/2012 16:36'!
platformName
	^  'iPhone'! !

Object subclass: #IPhoneTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'iPhone-Events'!
EventSensorConstants initialize!
InputEventSensor initialize!
